@@ -5,6 +5,10 @@
 
 #include "player.h"
 
+void *rtp_audio_receiver(void *arg);
+void *rtp_control_receiver(void *arg);
+void *rtp_timing_receiver(void *arg);
+
 void rtp_setup(SOCKADDR *remote, int controlport, int timingport, uint32_t active_remote,
                int *local_server_port, int *local_control_port, int *local_timing_port);
 void rtp_shutdown(void);
